package yukihane.inqubus.gui;

import java.awt.EventQueue;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.nio.file.Path;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.SortedSet;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JTextField;
import yukihane.inqubus.filewatch.FileWatch;
import yukihane.inqubus.filewatch.FileWatchUtil;

final class IdComboBox extends JComboBox<String> {

    private static final Logger logger = Logger.getLogger(IdComboBox.class.getName());
    private final Set<Path> alternates = new HashSet<>();
    private final FileWatch videoFileWatcher;

    IdComboBox(FileWatch videoFileWather) {
        super();
        this.videoFileWatcher = videoFileWather;
        setEditable(true);
        setSelectedIndex(-1);
        getEditorComponent().addKeyListener(new AutoCompleteAdapter());
        getEditorComponent().addFocusListener(new FocusListener() {

            @Override
            public void focusGained(FocusEvent e) {
                logger.log(Level.FINEST, "ID field focus gained");
                setAlternates(videoFileWatcher.getFiles());
            }

            @Override
            public void focusLost(FocusEvent e) {
                logger.log(Level.FINEST, "ID field focus lost");
            }
        });

    }

    JTextField getEditorComponent() {
        return (JTextField) getEditor().getEditorComponent();
    }

    void setAlternates(Collection<Path> alternates) {
        this.alternates.clear();
        this.alternates.addAll(alternates);
    }

    String getText() {
        return getEditorComponent().getText();
    }

    void setText(String str) {
        getEditorComponent().setText(str);
    }

    private class AutoCompleteAdapter extends KeyAdapter {

        @Override
        public void keyTyped(final KeyEvent e) {
            logger.log(Level.FINEST, "key typed");
            EventQueue.invokeLater(new Runnable() {

                @Override
                public void run() {
                    final String text = getText();
                    final SortedSet<String> alt = FileWatchUtil.getVideoIdsStartWith(alternates, text);
                    final String[] altArray = alt.toArray(new String[0]);
                    DefaultComboBoxModel<String> model = new DefaultComboBoxModel<>(altArray);
                    IdComboBox.this.setModel(model);
                    setSelectedIndex(-1);
                    setText(text);
                    IdComboBox.this.showPopup();
                }
            });
        }
    }
}